@@ -37,13 +37,14 @@ command line). This extension uses and wraps services of the keyring
 library.
 '''
 
-from mercurial import util
+from mercurial import util, sslutil
 from mercurial.i18n import _
 try:
     from mercurial.url import passwordmgr
 except:
     from mercurial.httprepo import passwordmgr
 from mercurial import mail
+from mercurial.mail import SMTPS, STARTTLS
 from mercurial import encoding
 from urllib2 import AbstractBasicAuthHandler, AbstractDigestAuthHandler
 
@@ -74,9 +75,24 @@ for blocked_module in [
     if blocked_module not in demandimport.ignore:
         demandimport.ignore.append(blocked_module)
 
+try:
+    is_demandimport_enabled = demandimport.isenabled
+except AttributeError:
+    # Mercurial < 2.9.1
+    try:
+        orig_demandimport = demandimport.demandimport
+    except AttributeError:
+        orig_demandimport = demandimport._demandimport
+    def is_demandimport_enabled():
+        return __import__ == orig_demandimport
+
+# Shut up warning about uninitialized logging for new keyring versions
+import logging
+logging.getLogger("keyring").addHandler(logging.NullHandler())
+
 # Temporarily disable demandimport to make the need of extending
 # the list above less likely.
-if __import__ == demandimport._import:
+if is_demandimport_enabled():
     demandimport.disable()
     try:
         import keyring
@@ -145,7 +161,7 @@ class PasswordStore(object):
         # keyring in general expects unicode. Mercurial provides "local" encoding. See #33
         password = encoding.fromlocal(password).decode('utf-8')
         keyring.set_password(KEYRING_SERVICE,
-                             pwdkey, 
+                             pwdkey,
                              password)
 
 password_store = PasswordStore()
@@ -449,22 +465,48 @@ def keyring_supported_smtp(ui, username):
     marked with #>>>>> and #<<<<< markers
     """
     local_hostname = ui.config('smtp', 'local_hostname')
-    s = smtplib.SMTP(local_hostname=local_hostname)
+    tls = ui.config('smtp', 'tls', 'none')
+    # backward compatible: when tls = true, we use starttls.
+    starttls = tls == 'starttls' or util.parsebool(tls)
+    smtps = tls == 'smtps'
+    if (starttls or smtps) and not util.safehasattr(socket, 'ssl'):
+        raise util.Abort(_("can't use TLS: Python SSL support not installed"))
     mailhost = ui.config('smtp', 'host')
     if not mailhost:
-        raise util.Abort(_('no [smtp]host in hgrc - cannot send mail'))
-    mailport = int(ui.config('smtp', 'port', 25))
+        raise util.Abort(_('smtp.host not configured - cannot send mail'))
+    verifycert = ui.config('smtp', 'verifycert', 'strict')
+    if verifycert not in ['strict', 'loose']:
+        if util.parsebool(verifycert) is not False:
+            raise util.Abort(_('invalid smtp.verifycert configuration: %s')
+                             % (verifycert))
+        verifycert = False
+    if (starttls or smtps) and verifycert:
+        sslkwargs = sslutil.sslkwargs(ui, mailhost)
+    else:
+        sslkwargs = {}
+    if smtps:
+        ui.note(_('(using smtps)\n'))
+        s = SMTPS(sslkwargs, local_hostname=local_hostname)
+    elif starttls:
+        s = STARTTLS(sslkwargs, local_hostname=local_hostname)
+    else:
+        s = smtplib.SMTP(local_hostname=local_hostname)
+    if smtps:
+        defaultport = 465
+    else:
+        defaultport = 25
+    mailport = util.getport(ui.config('smtp', 'port', defaultport))
     ui.note(_('sending mail: smtp host %s, port %s\n') %
             (mailhost, mailport))
     s.connect(host=mailhost, port=mailport)
-    if ui.configbool('smtp', 'tls'):
-        if not hasattr(socket, 'ssl'):
-            raise util.Abort(_("can't use TLS: Python SSL support "
-                               "not installed"))
-        ui.note(_('(using tls)\n'))
+    if starttls:
+        ui.note(_('(using starttls)\n'))
         s.ehlo()
         s.starttls()
         s.ehlo()
+    if (starttls or smtps) and verifycert:
+        ui.note(_('(verifying remote certificate)\n'))
+        sslutil.validator(ui, mailhost)(s.sock, verifycert == 'strict')
 
     #>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     stored = password = password_store.get_smtp_password(
